#This file is covered by the GNU General Public License.
#See the file COPYING.txt for more details.
# Une partie du code provient de l'extension Mozilla de Javi Dominguez
#version 2.5 du 22/11/2023

import sys
py3flag=True if sys.version[:1]=="3" else False
import globalCommands
from .py3compatibility import *  #pour filter
# from logHandler import loglog
import appModuleHandler
try: from nvdaBuiltin.appModules.firefox import AppModule
except ModuleNotFoundError: from appModuleHandler import AppModule
from NVDAObjects import NVDAObject
from NVDAObjects.IAccessible import IAccessible
try: from NVDAObjects.IAccessible.mozilla import Dialog, IAccessible
except ImportError: from NVDAObjects.IAccessible import Dialog, IAccessible
import scriptHandler
import controlTypes
if not hasattr(controlTypes, "Role"):
	setattr(controlTypes, "Role", type('Enum', (), dict(
	[(x.split("ROLE_")[1], getattr(controlTypes, x)) for x in dir(controlTypes) if x.startswith("ROLE_")])))
	setattr(controlTypes, "State", type('Enum', (), dict(
	[(x.split("STATE_")[1], getattr(controlTypes, x)) for x in dir(controlTypes) if x.startswith("STATE_")])))
	setattr(controlTypes, "role", type("role", (), {"_roleLabels": controlTypes.roleLabels}))
import api
import ui
import winUser
import speech
import gui
import wx
import core
from  datetime import datetime
from time import sleep
from threading import Timer
if py3flag: 
	from urllib  import parse  # pour Yannick
	from urllib.parse  import urlparse
else: from urlparse import urlparse
import re
from . import shared
from . shared import ia2, zRoleTexts,zeDevTools, getIA2Attribute 
from  . shared.ia2 import * 
from oleacc import *
from tones  import beep
from gui.settingsDialogs import SettingsDialog
from keyboardHandler import KeyboardInputGesture
import addonHandler

_curAddon = addonHandler.getCodeAddon()
# _scriptCategory = _curAddon.manifest['summary']
addonHandler.initTranslation()
nbTool=0

class TabsToolBar(object):  
	def __init__(self):
		self.oTabs= None
		try :
			if not AppModule.inMainWindow(self) : 
				print ("not in Main Window")
				return
			oForeground = api.getForegroundObject()
			oTabsToolbar = findObjectByIA2IdEx(oForeground, "TabsToolbar")
			if oTabsToolbar is None:
				print ("pas de oTabsToolbar")
				return
		except :
			return  
		self.oTabs = oTabsToolbar.firstChild #.next  # Onglets du navigateur Role.TOOLBAR
		if not self.oTabs.role==23 : self.oTabs = self.oTabs.next #TABCONTROL
		self.oTabsToolbar = oTabsToolbar
		
	def setFocus(self):
		if self.getSelectedTab() : self.getSelectedTab().IAccessibleObject.accSelect(1,0)

	def getSelectedTab(self):
		if not self.oTabs : return None
		index = self.getSelectedTabIndex()
		return self.oTabs.getChild(index)
	
	def getSelectedTabIndex(self):
		if not self.oTabs : return None
		i=0
		(o, id)  = accNavigate(self.oTabs.IAccessibleObject ,0,NAVDIR_FIRSTCHILD)
		while o:
			if o.accState(0) & STATE_SYSTEM_SELECTED:
				return i
			try:
				(o, id) = accNavigate(o,0,NAVDIR_NEXT)
				i= i+1
			except:
				o= None
		return None
	
	def getTabPosition(self):
		if self.oTabs is None:
			return ""
		count = self.oTabs.childCount -1
		if count>1:
			index = self.getSelectedTabIndex()
			return _("Tab {index} of {count}").format(index = index+1, count = count)    
		return ""
	
	def switchTo (self, oTab, speechMode):
		api.processPendingEvents()
		speech.cancelSpeech()
		# speech.speechMode = speechMode
		setSpeechMode(speechMode)
		oTab.doAction()

class Document(NVDAObject): 
	def _get_name(self):
		name=  super(Document, self)._get_name()
		global nbTool
		if nbTool>0 : name =None
		if name is not None:
			name = "%s %s" %(name, TabsToolBar().getTabPosition())
		return name
	
class Tab(NVDAObject):
	def initOverlayClass(self):
		self.bindGesture("kb:enter", "enter")
	def script_enter(self, gesture):
			KeyboardInputGesture.fromName("f6").send()
	__gestures ={
		"kb:enter": "enter"	}
		
class ToolsBarDialog(wx.Dialog):   
	def __init__(self, parent):
		super(ToolsBarDialog, self).__init__(parent, title="")
		mainSizer = wx.BoxSizer(wx.VERTICAL)
		self.listBox = wx.ListBox(self, wx.NewId(), style=wx.LB_SINGLE, size=(450, 300))
		mainSizer.Add(self.listBox, flag=wx.LEFT|wx.TOP|wx.RIGHT, border=10)
		buttonsSizer = wx.BoxSizer(wx.HORIZONTAL)
		goButtonID = wx.NewId()
		self.goButton = wx.Button(self, goButtonID, _("&Go"))
		buttonsSizer.Add(self.goButton, flag=wx.LEFT, border=20)
		optionsButtonID = wx.NewId()
		self.optionsButton = wx.Button(self, optionsButtonID, _("&Options"))
		buttonsSizer.Add(self.optionsButton, flag=wx.LEFT, border=20)
		self.Bind( wx.EVT_BUTTON, self.onOptionsButton, id=optionsButtonID)
		cancelButton = wx.Button(self, wx.ID_CANCEL, _("Annuler"))
		buttonsSizer.Add(cancelButton, flag=wx.LEFT, border=20)
		mainSizer.Add(buttonsSizer, flag=wx.ALL,  border=10)
		mainSizer.Fit(self)
		self.SetSizer(mainSizer)
		self.goButton.SetDefault()
		self.Bind( wx.EVT_BUTTON, self.onGoButton, id=goButtonID)
		self.Bind(wx.EVT_LISTBOX, self.onListBox, self.listBox)

	def onClose(self, evt):
		self.Destroy() 
		evt.Skip()	
	
	def update(self, items, title):
		self.items = items
		self.SetTitle(title)
		self.listBox.SetItems([item.name for item in self.items])
		try:
			selected = [controlTypes.State.SELECTED in item.states for item in self.items].index(True)
		except ValueError:
			selected = 0
		self.listBox.Select(selected)
		self.listBox.SetFocus()

	def getObjectFromList(self):
		index = self.listBox.GetSelections()
		if index is not None:
			obj = self.items[index[0]]
			return(obj)

	def onListBox(self, event):
		if self.getObjectFromList().role in [controlTypes.Role.BUTTON,controlTypes.Role.HEADING] :
			self.optionsButton.Enabled = False
		else:
			self.optionsButton.Enabled = True
	
	def forHeadings(self) :
		global nbTool
		def callback():
			nbTool =0
			scriptHandler.executeScript(globalCommands.commands.script_reportCurrentLine,None )
		try:	
			oldSpeechMode=getSpeechMode()
			setSpeechMode_off()
			scriptHandler.executeScript(globalCommands.commands.script_moveNavigatorObjectToMouse,None )
			scriptHandler.executeScript(globalCommands.commands.script_navigatorObject_moveFocus,None )
			sleep(0.2)
			scriptHandler.executeScript(globalCommands.commands.script_navigatorObject_moveFocus,None )
			sleep(0.2)
			setSpeechMode(oldSpeechMode)
			core.callLater(200,callback )
		except :
			print ("pb forHeadings")
			nbTool =0
			return
		
	def moveTo(self):
		speech.cancelSpeech()
		obj = self.getObjectFromList()
		if getIA2Attribute(obj,False,"id")in ["pageActionButton","PanelUI-menu-button","library-button","identity-box"]:
			obj0=api.getForegroundObject()
			if obj0.simpleFirstChild.role!=56 :
				KeyboardInputGesture.fromName("Tab").send()
				speech.cancelSpeech()
				ui.message (api.getFocusObject().name)
		
	def onGoButton(self, event):
		# self.Hide()
		self.Close()
		if self.moveMouseToObj():
			winUser.mouse_event(winUser.MOUSEEVENTF_LEFTDOWN,0,0,None,None)
			winUser.mouse_event(winUser.MOUSEEVENTF_LEFTUP,0,0,None,None)
		obj = self.getObjectFromList()
		if obj: 
			def move() :
				speech.cancelSpeech()
				# if nbTool==3 :
					# self.forHeadings()
				if nbTool==2 :
					if getIA2Attribute(obj,False,"id")in ["pageActionButton","PanelUI-menu-button","library-button","identity-box"]:
						self.moveTo()
				if nbTool==1 :	
					ui.message(obj.name)
			core.callLater(200,move)
		
	def onOptionsButton(self, event):
		# self.Hide()
		self.Close()
		if self.moveMouseToObj():
			winUser.mouse_event(winUser.MOUSEEVENTF_RIGHTDOWN,0,0,None,None)
			winUser.mouse_event(winUser.MOUSEEVENTF_RIGHTUP,0,0,None,None)

	def moveMouseToObj(self):
		obj = self.getObjectFromList()
		if obj:
			obj.scrollIntoView()
			api.moveMouseToNVDAObject(obj)
			api.setMouseObject(obj)
			return True
		return False  		

class AppModule(appModuleHandler.AppModule):
	tbDialog = None	
	scriptCategory = _("Firefox")
	
	def chooseNVDAObjectOverlayClasses(self, obj, clsList):
		obj.ia2ID=get_ia2ID(obj)
		if (obj.ia2ID and  obj.ia2ID=="main-window") or getIA2Attribute (obj,False,"tag")=="body" or obj.role == controlTypes.Role.DOCUMENT:
			clsList.insert(0, Document) 
		# check if focus is a tab  in tabsToolbar.
		if obj.role == controlTypes.Role.TAB:
			oParent = obj.parent
			if (oParent.role == controlTypes.Role.TABCONTROL and oParent.ia2ID == "tabbrowser-tabs") : 
				clsList.insert(0, Tab)			
			
	def script_goToTabsToolbar(self, gesture):  
		TabsToolBar().setFocus()
	script_goToTabsToolbar.__doc__ = _("Move the focus to tabs toolbar")
	script_goToTabsToolbar.category = _("Firefox")  
	
	def script_copyLink(self, gesture):  
		ffVersion = int(self.productVersion.split(".")[0])		
		if  ffVersion > 89:
			return self.ReadsLinkAddressUnderCursor()
		# beep(333,66)
		if api.getFocusObject() :
			try:
				obj=api.getFocusObject()
				if obj.value:
					api.copyToClip(obj.value)
					ui.message(_("link %s copied in the clipboard.") %obj.name)
				else : 
					ui.message (_("no link selected"))
			except NameError:
				# print ("pb script_copyLink")
				return
	script_copyLink.__doc__ = _("Copies selected link to clipboard.")	
	script_copyLink.category = _("Firefox")  	
	
	def script_goToLibButton(self, gesture): 
		if not self.inMainWindow() : return ui.message(_("Not available here"))
		try :
			path = (("id", "nav-bar"), ("id", "library-button"))
			oLibButton = shared.searchObject(path)
			if oLibButton is None:
				return
			oLibButton.doAction()
			wx.CallLater(300,ui.message ,_("Library button, type downArrow") )
		except :
			return  
	script_goToTabsToolbar.__doc__ = _("Move the focus to tabs toolbar")
	script_goToTabsToolbar.category = _("Firefox")  		
	
	def script_tabMovement(self, gesture):
		gesture.send()
		global nbTool
		nbTool=0
		obj=api.getForegroundObject()
		basestring = str
		title=obj.name
		if not isinstance(title,basestring) or not title or title.isspace():
			title=obj.appModule.appName  if obj.appModule else None
			if not isinstance(title,basestring) or not title or title.isspace():
				return
		ui.message(title) 

	def event_alert(self, obj, nextHandler):
		try:
			if obj.IA2Attributes["id"] == "customizationui-widget-panel":
				# Firefox add-on panel
				speech.cancelSpeech()
				api.setFocusObject(obj.simpleFirstChild)
				nextHandler()
				return
		except (KeyError, AttributeError):
			pass
		if re.match(".*notification.*|.*alert.*", ";".join(obj.IA2Attributes.values()).lower()):
			try:
				isPopup = obj.IA2Attributes["id"] == "notification-popup"
			except (KeyError, AttributeError):
				isPopup = False
			if self.inMainWindow() and isPopup:
				if shared.focusAlertPopup(obj):
					Timer(0.75, nextHandler)
					return
			alertText = shared.getAlertText(obj)
			# Appends the domain of the page where it was when the alert originated.
			path = (("id", "nav-bar"), ("id", "urlbar"), ("class", "urlbar-input textbox-input",))
			url = shared.searchObject(path)
			if url and url.value:
				url = url.value if "://" in url.value else "None://"+url.value
				domain = urlparse(url).hostname if url else ""
				if domain and domain not in alertText: 
					alertText = "%s\n\n%s" % (alertText, domain)
					shared.notificationsDialog.registerFirefoxNotification((datetime.now(), alertText))  #<<<<
		nextHandler()

	def script_status(self, gesture):
		if not self.inMainWindow(): return ui.message(_("Not available here"))
		group = shared.searchAmongTheChildren(("tag", "tabpanels"), api.getForegroundObject())
		if group:
			for propertyPage in filter(lambda o: o.role == controlTypes.ROLE_PROPERTYPAGE, group.children):
				try:
					obj = filter(lambda o: o.role == controlTypes.ROLE_STATUSBAR, propertyPage.children)[0]
					break
				except IndexError:
					pass
			try:
				ui.message(obj.firstChild.name)
			except NameError:
				#TRANSLATORS: message spoken when there is no status bar in Firefox
				ui.message (_("Status bar not found"))
			else:
			# if obj :
				if scriptHandler.getLastScriptRepeatCount() == 1:
					if api.copyToClip(obj.firstChild.name):
						#TRANSLATORS: message spoken when an item hast just been copied to the clipboard
						ui.message(_("Copied to clipboard"))
			return
		ui.message (_("Status bar not found"))
	script_status.__doc__ = _("Reads the status bar. If pressed twice quickly, copies it to clipboard.")
	script_status.category = _("Firefox") 

	def script_url(self, gesture):
		if not self.inMainWindow():
			ui.message(_("Not available here"))
			return
		ffVersion = int(self.productVersion.split(".")[0])
		if  ffVersion > 89:
			return self.reportConstantIdentifier()
		# beep(333,66)
		if ffVersion < 70:
			path = (("id", "nav-bar"), ("id", "urlbar"), ("id", "identity-box",))
		elif ffVersion < 76:
			path = (("id", "nav-bar"), ("id", "identity-box"), ("id", "identity-icon"))
		elif ffVersion < 87:
			path = (("id", "nav-bar"), ("id", "identity-box"))
		else:
			path = (("id", "nav-bar"), ("id", "identity-icon-box"))     #("id","urlbar-input"))  #  ("class","urlbar-input-box"),
		secInfoButton = shared.searchObject(path)
		if secInfoButton:
			securInfo = secInfoButton.description # This has changed in FF 57. Keeping this line for compatibility with earlier versions.
			try: # This one is for FF 57 and later.
				try: # FF 57 to 69
					securInfo = secInfoButton.firstChild.next.name if secInfoButton.firstChild.next.IA2Attributes["id"] == "connection-icon" else ""
					if securInfo:
						owner = " ".join([o.name for o in filter(lambda o: o.role == controlTypes.Role.STATICTEXT, secInfoButton.recursiveDescendants)])
						securInfo = "%s, %s" % (owner, securInfo) if owner else securInfo
				except AttributeError: # FF 70 and above 
					securInfo = secInfoButton.name
			except:
				pass
			securInfo  = _("Insecure connection") if not securInfo   else securInfo  
			if ffVersion < 76:
				url = secInfoButton.next.value if ffVersion < 70 else secInfoButton.parent.next.firstChild.value
				ui.message("%s (%s)" % (url, securInfo))
			elif ffVersion < 87:
				print (secInfoButton.next.firstChild.value)
				url = secInfoButton.next.firstChild.value
				if url=="" : url=u"La barre d'adresses est vide"
				else :url = secInfoButton.next.firstChild.value
				ui.message(url)
			else:
				# url = secInfoButton.value
				try :
					url = secInfoButton.next.firstChild.value
					msg=(url if url else u"La barre d'adresses est vide")
					ui.message(msg)
				except :
					ui.message(u"La barre d'adresses est vide")
			if scriptHandler.getLastScriptRepeatCount() == 1:
				if api.copyToClip(url):
					ui.message(_("Copied to clipboard"))
			return
		ui.message (_("Address not found"))
	script_url.__doc__ = _("Reads the page address. If pressed twice quickly, copies it to clipboard.")

	def script_toolsF8(self, gesture):
		self.showToolsDialog(1)
	script_toolsF8.__doc__ = _("Shows a list of opened tabs.")
	script_toolsF8.category = _("Firefox") 

	def script_toolsF6(self, gesture):
		self.showToolsDialog(2)
	script_toolsF6.__doc__ = _("Shows buttons of tool bar.")
	script_toolsF6.category = _("Firefox") 
	"""def script_toolsF4(self, gesture):
		self.showToolsDialog(3)
	script_toolsF4.__doc__ = _("Shows buttons of tool bar.") """
	
	def showToolsDialog(self, nb):  #different de 16
		if not self.inMainWindow() and api.getForegroundObject().appModule.productName != "NVDA":  return ui.message(_("Not available here"))
		# if not self.tbDialog: return
		global nbTool 
		nbTool=nb
		if not self.tbDialog:
			self.tbDialog = ToolsBarDialog(gui.mainFrame) 
		if nb==1:
			items, title = self.getTabsDialog() 
		elif nb==2:
			items, title = self.getButtonsDialog() 
		# elif nb==3:
			# items, title = self.getHeadersDialog()
		if items:
			self.tbDialog.update(items, title)
			if not self.tbDialog.IsShown():
				gui.mainFrame.prePopup()
				self.tbDialog.CenterOnScreen()
				self.tbDialog.Show(True)
				gui.mainFrame.postPopup()
			return
		ui.message (_("No element found"))

	def script_notifications(self, gesture):
		obj = api.getForegroundObject().simpleFirstChild
		if obj.role == controlTypes.Role.ALERT:
			if api.getFocusObject().parent == obj: # Already focused
				speech.speakObject(obj)
				speech.speakObject(api.getFocusObject())
				return
			if shared.focusAlertPopup(obj):
				speech.speakObject(api.getFocusObject())
				return
		if not shared.notificationsDialog.isEmpty():
			if scriptHandler.getLastScriptRepeatCount() == 1:
				# Gesture repeated twice shows the complete history in a dialog box.
				# shared.notificationsDialog.firefoxPage()
				return
			else:
				# Gesture once says the last notification
				if shared.notificationsDialog.history["Firefox"]:
					timestamp, message = shared.notificationsDialog.history["Firefox"][0]
					ui.message("%s, %s" % (shared.elapsedFromTimestamp(timestamp), message))
					return
		# There is no notification in Firefox or Thunderbird
		ui.message(_("There is no notification"))
	script_notifications.__doc__ = _("Reads the last notification and it takes the system focus to it if it is possible. By pressing two times quickly shows the history of notifications.")
	script_notifications.category = _("Firefox") 

	def script_focusDocument(self, gesture):  #different 16
		if not self.inMainWindow(): return ui.message(_("Not available here"))
		global nbTool
		if nbTool>0 :nbTool=0		
		if not api.getForegroundObject() : 
			# beep(222,66)  
			return
		fg=api.getForegroundObject()
		group = shared.searchAmongTheChildren(("tag", "tabpanels"), fg)
		if not group : 
			return
		if group:
			for propertyPage in filter(lambda o: o.role == controlTypes.Role.PROPERTYPAGE, group.children):
				try:
					doc = filter(lambda o: o.role == controlTypes.Role.INTERNALFRAME and controlTypes.State.FOCUSABLE in o.states , propertyPage.children)[0].children[0]
					break
				except IndexError:
					pass
			try:
				api.setFocusObject(doc)
				try:
					doc.treeInterceptor.passThrough = False
				except AttributeError:
					pass
				ui.message("%s %s" % (controlTypes.role._roleLabels[doc.role], doc.name))			
			except NameError:
				pass
	script_focusDocument.__doc__ = _("Brings the focus to the document")
	script_focusDocument.category = _("Firefox") 

	def getTabsDialog(self):
		tabs = ""
		fg = api.getForegroundObject()
		for toolBar in filter(lambda o: o.role == controlTypes.Role.TOOLBAR, fg.children):
			try:
				tabControl = filter(lambda o: o.role == controlTypes.Role.TABCONTROL, toolBar.children)[0]
			except IndexError:
				pass
			else:
				tabs = filter(lambda o: o.role == controlTypes.Role.TAB, tabControl.children)
		if len(tabs)==1 : return tabs, "%d %s" % (len(tabs), " onglet ouvert")  #(u"Un onglet ouvert")
		else : return tabs, "%d %s" % (len(tabs), " onglets ouverts")

	def getButtonsDialog(self):
		fg = api.getForegroundObject()
		buttons = []
		for toolBar in filter(lambda o: o.role == controlTypes.Role.TOOLBAR and "id" in o.IA2Attributes and o.IA2Attributes["id"] != "TabsToolbar", fg.children):
			buttons = buttons + filter(lambda o: o.role == controlTypes.Role.BUTTON and controlTypes.State.OFFSCREEN not in o.states, toolBar.children)
		return buttons, _("Tool Bar Buttons")
		
	def getHeadersDialog(self): #pb if several tabs, several propertypage   # en plus !!! appel par showToolsDialog(
		fg = api.getForegroundObject()
		nam=fg.name
		headings = []
		group = shared.searchAmongTheChildren(("tag", "tabpanels"), api.getForegroundObject())
		if not group : 
			# beep(666,66) 
			return
		if group:
			for propertyPage in filter(lambda o: o.role == controlTypes.Role.PROPERTYPAGE, group.children):
				try:
					doc = filter(lambda o: o.role == controlTypes.Role.INTERNALFRAME and controlTypes.State.FOCUSABLE in o.states, propertyPage.children)[0].children[0]
					break
				except IndexError:
					pass
			try:
				api.setFocusObject(doc)
				try:
					doc.treeInterceptor.passThrough = True  #False
				except AttributeError:
					pass
			except NameError:
				print ("Not found") 
				return
			doc=doc.parent
			for docu in filter(lambda o: o.role == controlTypes.Role.DOCUMENT , doc.children):  
				headings = headings + filter(lambda o: o.role == controlTypes.Role.HEADING, docu.children)	#Role.HEADING #Role.LINK
		return headings, _("List of %d headings" )% len(headings)

	def inMainWindow(self):
		fg = api.getForegroundObject()
		try:
			if fg.IA2Attributes["id"] == "main-window":
				return True
		except (AttributeError, KeyError):
			pass
		# As of Firefox 72, the IA2Attributes["id"]:"main-window" is no longer used
		try:
			if fg.IA2Attributes["tag"] == "body":
				return True
		except (AttributeError, KeyError):
			pass
			
	def script_listObjects (self, gesture) :
		index=None
		try:
			# index = int(gesture.keyName[-1])-1
			index = (gesture.keyName[-1])-1
			print (index)
		except AttributeError:
			# index = int(gesture.mainKeyName[-1])-1
			pass
		# twice = True if (scriptHandler.getLastScriptRepeatCount() == 1 ) else False  #and index == self.lastIndex
		
		twice = True
		if twice==False :   
			self.showToolsDialog(1)  #boutons

		else : 
			# wx.CallAfter(KeyboardInputGesture.fromName("escape").send)
			wx.CallLater(1000,self.showToolsDialog,2)  #onglets
		# wx.CallAfter(updateAddon.checkUpdate, False,twice)  
		# wx.CallLater(100,applilist.ElementsListDialog.run, twice)
		# self.showToolsDialog(1)
		self.lastIndex = index
		
	def reportConstantIdentifier(self):
		repeatCount = scriptHandler.getLastScriptRepeatCount()
		treeInterceptor = api.getFocusObject().treeInterceptor
		text = "%s copié dans le presse papier"
		url = treeInterceptor and treeInterceptor.documentConstantIdentifier
		if url is None:
			text ="Impossible de récupérer l'URL"
		elif repeatCount == 0:
			text = parse.urlparse(url).hostname or parse.urlparse(url).netloc or url
			text = text[4:] if text.startswith("www.") else text
			text += ". %s" % url
		elif repeatCount == 1:
			windowName = api.getForegroundObject().name[:-18]
			text = (
				(api.copyToClip(windowName + "\r\n" + url) and (text % "adresse et titre"))
				or "Impossible de copier l'adresse  et le titre"
			)
		else:
			text = (
				(api.copyToClip(url) and (text % "adresse"))  
				or  "impossible de copier l'URL"
			)
		ui.message(text)
	# script_reportConstantIdentifier.__doc__ = u"Un appui donne le nom de domaine et son adresse. Deux appuis copient dans le presse papier le titre du document et son adresse.Trois appuis copient uniquement l'adresse."
	# script_reportConstantIdentifier.category = _("Firefox") 
		
	def ReadsLinkAddressUnderCursor(self):
		obj = api.getFocusObject().treeInterceptor.selection.NVDAObjectAtStart
		scriptRepeatCount = bool(scriptHandler.getLastScriptRepeatCount())
		if obj.role == controlTypes.Role.LINK:
			if scriptRepeatCount:
				text = (
					(
						api.copyToClip(
							"[%s](%s)" % (obj.name.strip(), obj.value)
						)
						and "texte et lien copiés."
					)
					or "Impossible de copier l'adresse"
				)
			else:
				text = obj.value
		else:
			text = "Aucun lien sous le curseur"
		ui.message(text)
		
	def script_test(self,gesture):
		ui.message ("test local")	
		obj = api.getFocusObject()
		obj.reportFocus()
		# ui.message (str(self.inMainWindow()))			
	__gestures = {
		"kb:NVDA+F4": "url",
		"kb(desktop):NVDA+shift+A": "url",
		"kb(laptop):NVDA+Control+A": "url",
		"kb(desktop):NVDA+End": "status",
		"kb(laptop):NVDA+Shift+End": "status",		
		"kb:control+shift+pageUp": "tabMovement",
		"kb:control+shift+pageDown": "tabMovement",
		"kb:NVDA+Control+N": "notifications",
		"kb:control+shift+L":"copyLink",
		"kb:NVDA+F6": "toolsF6",		
		"kb:NVDA+control+F6": "goToLibButton",		
		"kb:NVDA+F8": "toolsF8",
		"kb:control+F8": "goToTabsToolbar", 
		# "kb:control+shift+alt+!": "test"  
	}


